@@ -1,5 +1,5 @@
 /*
- * Copyright 2016-2018, Intel Corporation
+ * Copyright 2016-2020, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -38,13 +38,22 @@
 #ifndef LIBPMEMOBJ_CPP_POOL_HPP
 #define LIBPMEMOBJ_CPP_POOL_HPP
 
-#include <stddef.h>
+#include <cstddef>
+#include <functional>
+#include <mutex>
 #include <string>
 #include <sys/stat.h>
+#include <typeindex>
+#include <unordered_map>
+#include <vector>
 
 #include <libpmemobj++/detail/common.hpp>
-#include <libpmemobj++/detail/pexceptions.hpp>
+#include <libpmemobj++/detail/ctl.hpp>
+#include <libpmemobj++/detail/pool_data.hpp>
 #include <libpmemobj++/p.hpp>
+#include <libpmemobj++/persistent_ptr_base.hpp>
+#include <libpmemobj++/pexceptions.hpp>
+#include <libpmemobj/atomic_base.h>
 #include <libpmemobj/pool_base.h>
 
 namespace pmem
@@ -129,7 +138,10 @@ public:
 		pmemobjpool *pop = pmemobj_open(path.c_str(), layout.c_str());
 #endif
 		if (pop == nullptr)
-			throw pool_error("Failed opening pool");
+			throw pmem::pool_error("Failed opening pool")
+				.with_pmemobj_errormsg();
+
+		pmemobj_set_user_data(pop, new detail::pool_data);
 
 		return pool_base(pop);
 	}
@@ -162,7 +174,10 @@ public:
 						  size, mode);
 #endif
 		if (pop == nullptr)
-			throw pool_error("Failed creating pool");
+			throw pmem::pool_error("Failed creating pool")
+				.with_pmemobj_errormsg();
+
+		pmemobj_set_user_data(pop, new detail::pool_data);
 
 		return pool_base(pop);
 	}
@@ -206,7 +221,10 @@ public:
 	{
 		pmemobjpool *pop = pmemobj_openW(path.c_str(), layout.c_str());
 		if (pop == nullptr)
-			throw pool_error("Failed opening pool");
+			throw pmem::pool_error("Failed opening pool")
+				.with_pmemobj_errormsg();
+
+		pmemobj_set_user_data(pop, new detail::pool_data);
 
 		return pool_base(pop);
 	}
@@ -235,7 +253,10 @@ public:
 		pmemobjpool *pop = pmemobj_createW(path.c_str(), layout.c_str(),
 						   size, mode);
 		if (pop == nullptr)
-			throw pool_error("Failed creating pool");
+			throw pmem::pool_error("Failed creating pool")
+				.with_pmemobj_errormsg();
+
+		pmemobj_set_user_data(pop, new detail::pool_data);
 
 		return pool_base(pop);
 	}
@@ -269,6 +290,14 @@ public:
 		if (this->pop == nullptr)
 			throw std::logic_error("Pool already closed");
 
+		auto *user_data = static_cast<detail::pool_data *>(
+			pmemobj_get_user_data(this->pop));
+
+		if (user_data->initialized.load())
+			user_data->cleanup();
+
+		delete user_data;
+
 		pmemobj_close(this->pop);
 		this->pop = nullptr;
 	}
@@ -298,7 +327,8 @@ public:
 	}
 
 	/**
-	 * Performs persist operation on a given persistent object.
+	 * Performs persist operation on a given persistent pointer.
+	 * Persist is not performed on the object referenced by this pointer.
 	 *
 	 * @param[in] ptr Persistent pointer to object
 	 */
@@ -386,7 +416,7 @@ public:
 		return pmemobj_memset_persist(this->pop, dest, c, len);
 	}
 
-	/*
+	/**
 	 * Gets the C style handle to the pool.
 	 *
 	 * Necessary to be able to use the pool with the C API.
@@ -405,6 +435,33 @@ public:
 		return pool_base::handle();
 	}
 
+	/**
+	 * Starts defragmentation using selected pointers within this pool.
+	 *
+	 * @param[in] ptrv pointer to contiguous space containing
+	 *	persistent_ptr's for defrag.
+	 * @param[in] oidcnt number of persistent_ptr's passed (in ptrv).
+	 * @return result struct containing a number of relocated and total
+	 *	processed objects.
+	 *
+	 * @throw pmem::defrag_error when a failure during defragmentation
+	 *	occurs. Even if this error is thrown, some of the objects could
+	 *	have been relocated, see defrag_error.result for summary stats.
+	 */
+	pobj_defrag_result
+	defrag(persistent_ptr_base **ptrv, size_t oidcnt)
+	{
+		pobj_defrag_result result;
+		int ret = pmemobj_defrag(this->pop, (PMEMoid **)ptrv, oidcnt,
+					 &result);
+
+		if (ret != 0)
+			throw defrag_error(result, "Defragmentation failed")
+				.with_pmemobj_errormsg();
+
+		return result;
+	}
+
 protected:
 	/* The pool opaque handle */
 	PMEMobjpool *pop;
@@ -425,6 +482,10 @@ protected:
  * for operations on pmemobj pools. The template parameter defines the
  * type of the root object within the pool. The typical usage example would be:
  * @snippet doc_snippets/pool.cpp pool_example
+ *
+ * This API should not be mixed with C API. For example explicitly calling
+ * pmemobj_set_user_data(pop) on pool which is handled by C++ pool object
+ * is undefined behaviour.
  */
 template <typename T>
 class pool : public pool_base {
@@ -473,6 +534,114 @@ public:
 	{
 	}
 
+	/**
+	 * Query libpmemobj state at pool scope.
+	 *
+	 * @param[in] name name of entry point
+	 *
+	 * @returns variable representing internal state
+	 *
+	 * For more details, see:
+	 * https://pmem.io/pmdk/manpages/linux/master/libpmemobj/pmemobj_ctl_get.3
+	 */
+	template <typename M>
+	M
+	ctl_get(const std::string &name)
+	{
+		return ctl_get_detail<M>(pop, name);
+	}
+
+	/**
+	 * Modify libpmemobj state at pool scope.
+	 *
+	 * @param[in] name name of entry point
+	 * @param[in] arg extra argument
+	 *
+	 * @returns copy of arg, possibly modified by query
+	 *
+	 * For more details, see:
+	 * https://pmem.io/pmdk/manpages/linux/master/libpmemobj/pmemobj_ctl_get.3
+	 */
+	template <typename M>
+	M
+	ctl_set(const std::string &name, M arg)
+	{
+		return ctl_set_detail(pop, name, arg);
+	}
+
+	/**
+	 * Execute function at pool scope.
+	 *
+	 * @param[in] name name of entry point
+	 * @param[in] arg extra argument
+	 *
+	 * @returns copy of arg, possibly modified by query
+	 *
+	 * For more details, see:
+	 * https://pmem.io/pmdk/manpages/linux/master/libpmemobj/pmemobj_ctl_get.3
+	 */
+	template <typename M>
+	M
+	ctl_exec(const std::string &name, M arg)
+	{
+		return ctl_exec_detail(pop, name, arg);
+	}
+
+#ifdef _WIN32
+	/**
+	 * Query libpmemobj state at pool scope.
+	 *
+	 * @param[in] name name of entry point
+	 *
+	 * @returns variable representing internal state
+	 *
+	 * For more details, see:
+	 * https://pmem.io/pmdk/manpages/linux/master/libpmemobj/pmemobj_ctl_get.3
+	 */
+	template <typename M>
+	M
+	ctl_get(const std::wstring &name)
+	{
+		return ctl_get_detail<M>(pop, name);
+	}
+
+	/**
+	 * Modify libpmemobj state at pool scope.
+	 *
+	 * @param[in] name name of entry point
+	 * @param[in] arg extra argument
+	 *
+	 * @returns copy of arg, possibly modified by query
+	 *
+	 * For more details, see:
+	 * https://pmem.io/pmdk/manpages/linux/master/libpmemobj/pmemobj_ctl_get.3
+	 */
+	template <typename M>
+	M
+	ctl_set(const std::wstring &name, M arg)
+	{
+		return ctl_set_detail(pop, name, arg);
+	}
+
+	/**
+	 * Execute function at pool scope.
+	 *
+	 * @param[in] name name of entry point
+	 * @param[in] arg extra argument
+	 *
+	 * @returns copy of arg, possibly modified by query
+	 *
+	 * For more details, see:
+	 * https://pmem.io/pmdk/manpages/linux/master/libpmemobj/pmemobj_ctl_get.3
+	 */
+	template <typename M>
+	M
+	ctl_exec(const std::wstring &name, M arg)
+	{
+		return ctl_exec_detail(pop, name, arg);
+	}
+#endif
+
 	/**
 	 * Retrieves pool's root object.
 	 *
@@ -482,7 +651,7 @@ public:
 	root()
 	{
 		if (pop == nullptr)
-			throw pool_error("Invalid pool handle");
+			throw pmem::pool_error("Invalid pool handle");
 
 		persistent_ptr<T> root = pmemobj_root(this->pop, sizeof(T));
 		return root;
@@ -614,6 +783,114 @@ public:
 #endif
 };
 
+/**
+ * Query libpmemobj state at global scope.
+ *
+ * @param[in] name name of entry point
+ *
+ * @returns variable representing internal state
+ *
+ * For more details, see:
+ * https://pmem.io/pmdk/manpages/linux/master/libpmemobj/pmemobj_ctl_get.3
+ */
+template <typename T>
+T
+ctl_get(const std::string &name)
+{
+	return ctl_get_detail<T>(nullptr, name);
+}
+
+/**
+ * Modify libpmemobj state at global scope.
+ *
+ * @param[in] name name of entry point
+ * @param[in] arg extra argument
+ *
+ * @returns copy of arg, possibly modified by query
+ *
+ * For more details, see:
+ * https://pmem.io/pmdk/manpages/linux/master/libpmemobj/pmemobj_ctl_get.3
+ */
+template <typename T>
+T
+ctl_set(const std::string &name, T arg)
+{
+	return ctl_set_detail(nullptr, name, arg);
+}
+
+/**
+ * Execute function at global scope.
+ *
+ * @param[in] name name of entry point
+ * @param[in] arg extra argument
+ *
+ * @returns copy of arg, possibly modified by query
+ *
+ * For more details, see:
+ * https://pmem.io/pmdk/manpages/linux/master/libpmemobj/pmemobj_ctl_get.3
+ */
+template <typename T>
+T
+ctl_exec(const std::string &name, T arg)
+{
+	return ctl_exec_detail(nullptr, name, arg);
+}
+
+#ifdef _WIN32
+/**
+ * Query libpmemobj state at global scope.
+ *
+ * @param[in] name name of entry point
+ *
+ * @returns variable representing internal state
+ *
+ * For more details, see:
+ * https://pmem.io/pmdk/manpages/linux/master/libpmemobj/pmemobj_ctl_get.3
+ */
+template <typename T>
+T
+ctl_get(const std::wstring &name)
+{
+	return ctl_get_detail<T>(nullptr, name);
+}
+
+/**
+ * Modify libpmemobj state at global scope.
+ *
+ * @param[in] name name of entry point
+ * @param[in] arg extra argument
+ *
+ * @returns copy of arg, possibly modified by query
+ *
+ * For more details, see:
+ * https://pmem.io/pmdk/manpages/linux/master/libpmemobj/pmemobj_ctl_get.3
+ */
+template <typename T>
+T
+ctl_set(const std::wstring &name, T arg)
+{
+	return ctl_set_detail(nullptr, name, arg);
+}
+
+/**
+ * Execute function at global scope.
+ *
+ * @param[in] name name of entry point
+ * @param[in] arg extra argument
+ *
+ * @returns copy of arg, possibly modified by query
+ *
+ * For more details, see:
+ * https://pmem.io/pmdk/manpages/linux/master/libpmemobj/pmemobj_ctl_get.3
+ */
+template <typename T>
+T
+ctl_exec(const std::wstring &name, T arg)
+{
+	return ctl_exec_detail(nullptr, name, arg);
+}
+#endif
+
 } /* namespace obj */
 
 } /* namespace pmem */
