@@ -14,11 +14,11 @@
 
 #include "unittest.hpp"
 
-#include <libpmemobj++/experimental/string.hpp>
+#include <libpmemobj++/container/string.hpp>
 
 namespace nvobj = pmem::obj;
-namespace pmem_exp = nvobj::experimental;
-using C = pmem_exp::string;
+
+using C = pmem::obj::string;
 
 struct root {
 	nvobj::persistent_ptr<C> s1, s2;
