@@ -1,5 +1,5 @@
 /*
- * Copyright 2016-2019, Intel Corporation
+ * Copyright 2016-2020, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -72,7 +72,7 @@ uncaught_exceptions() noexcept
 #include <libpmemobj++/transaction.hpp>
 
 #define LAYOUT "cpp"
-#define POOL_SZIE PMEMOBJ_MIN_POOL
+#define POOL_SIZE 2 * PMEMOBJ_MIN_POOL
 
 namespace nvobj = pmem::obj;
 
@@ -544,7 +544,7 @@ test_tx_throw_no_abort_scope(nvobj::pool<root> &pop)
 	UT_ASSERT(rootp->pfoo == nullptr);
 	UT_ASSERT(rootp->parr == nullptr);
 
-	/* commiting non-existent transaction should fail with an exception */
+	/* committing non-existent transaction should fail with an exception */
 	exception_thrown = false;
 	try {
 		nvobj::transaction::commit();
@@ -792,7 +792,7 @@ test_tx_automatic_destructor_throw(nvobj::pool<root> &pop)
  * pmemobj_tx_add_range_direct() failed.
  * 3) Check if assigning value to pmem object is valid under pmemcheck when
  * object was snapshotted beforehand.
- * 4) Check if snapshotted value was rolled back in case of transacion abort.
+ * 4) Check if snapshotted value was rolled back in case of transaction abort.
  */
 void
 test_tx_snapshot(nvobj::pool<root> &pop)
@@ -808,6 +808,8 @@ test_tx_snapshot(nvobj::pool<root> &pop)
 	try {
 		nvobj::transaction::snapshot<char>(parr.get(), 5);
 		UT_ASSERT(0);
+	} catch (std::bad_alloc &) {
+		UT_ASSERT(0);
 	} catch (pmem::transaction_error &) {
 		exception_thrown = true;
 	} catch (...) {
@@ -819,10 +821,85 @@ test_tx_snapshot(nvobj::pool<root> &pop)
 	try {
 		nvobj::transaction::run(pop, [&] {
 			nvobj::transaction::snapshot<char>(parr.get(),
-							   POOL_SZIE);
+							   POOL_SIZE);
 		});
 		UT_ASSERT(0);
-	} catch (pmem::transaction_error &) {
+	} catch (pmem::transaction_error &e) {
+		(void)e.what();
+		exception_thrown = true;
+	} catch (...) {
+		UT_ASSERT(0);
+	}
+	UT_ASSERT(exception_thrown);
+
+	nvobj::persistent_ptr<char[]> p1;
+
+	try {
+		nvobj::transaction::run(pop, [&] {
+			p1 = nvobj::make_persistent<char[]>(POOL_SIZE / 2);
+		});
+	} catch (...) {
+		UT_ASSERT(0);
+	}
+
+	/* OOM handling */
+	exception_thrown = false;
+	try {
+		nvobj::transaction::run(pop, [&] {
+			nvobj::transaction::snapshot<char>(p1.get(),
+							   POOL_SIZE / 2);
+		});
+		UT_ASSERT(0);
+	} catch (std::bad_alloc &e) {
+		(void)e.what();
+		exception_thrown = true;
+	} catch (...) {
+		UT_ASSERT(0);
+	}
+	UT_ASSERT(exception_thrown);
+
+	/* OOM handling */
+	exception_thrown = false;
+	try {
+		nvobj::transaction::run(pop, [&] {
+			nvobj::transaction::snapshot<char>(p1.get(),
+							   POOL_SIZE / 2);
+		});
+		UT_ASSERT(0);
+	} catch (pmem::transaction_alloc_error &e) {
+		(void)e.what();
+		exception_thrown = true;
+	} catch (...) {
+		UT_ASSERT(0);
+	}
+	UT_ASSERT(exception_thrown);
+
+	/* OOM handling */
+	exception_thrown = false;
+	try {
+		nvobj::transaction::run(pop, [&] {
+			pmem::detail::conditional_add_to_tx<char>(
+				p1.get(), POOL_SIZE / 2);
+		});
+		UT_ASSERT(0);
+	} catch (pmem::transaction_alloc_error &e) {
+		(void)e.what();
+		exception_thrown = true;
+	} catch (...) {
+		UT_ASSERT(0);
+	}
+	UT_ASSERT(exception_thrown);
+
+	/* OOM handling */
+	exception_thrown = false;
+	try {
+		nvobj::transaction::run(pop, [&] {
+			pmem::detail::conditional_add_to_tx<char>(
+				p1.get(), POOL_SIZE / 2);
+		});
+		UT_ASSERT(0);
+	} catch (std::bad_alloc &e) {
+		(void)e.what();
 		exception_thrown = true;
 	} catch (...) {
 		UT_ASSERT(0);
@@ -854,6 +931,205 @@ test_tx_snapshot(nvobj::pool<root> &pop)
 		UT_ASSERT(0);
 	}
 }
+
+void
+tx_with_callbacks(int &cb_oncommit_called, int &cb_onabort_called,
+		  int &cb_finally_called, int &cb_work_called)
+{
+	nvobj::transaction::register_callback(nvobj::transaction::stage::work,
+					      [&] { cb_work_called++; });
+
+	nvobj::transaction::register_callback(
+		nvobj::transaction::stage::oncommit, [&] {
+			UT_ASSERT(cb_work_called == 2);
+			cb_oncommit_called++;
+		});
+
+	nvobj::transaction::register_callback(nvobj::transaction::stage::work,
+					      [&] { cb_work_called++; });
+
+	nvobj::transaction::register_callback(
+		nvobj::transaction::stage::onabort,
+		[&] { cb_onabort_called++; });
+
+	nvobj::transaction::register_callback(
+		nvobj::transaction::stage::finally,
+		[&] { cb_finally_called++; });
+
+	UT_ASSERT(cb_work_called == 0);
+	UT_ASSERT(cb_oncommit_called == 0);
+	UT_ASSERT(cb_onabort_called == 0);
+	UT_ASSERT(cb_finally_called == 0);
+}
+
+void
+test_tx_callback(nvobj::pool<root> &pop)
+{
+	int cb_oncommit_called = 0;
+	int cb_onabort_called = 0;
+	int cb_finally_called = 0;
+	int cb_work_called = 0;
+
+	try {
+		tx_with_callbacks(cb_oncommit_called, cb_onabort_called,
+				  cb_finally_called, cb_work_called);
+
+		UT_ASSERT(0);
+	} catch (pmem::transaction_scope_error &) {
+	} catch (...) {
+		UT_ASSERT(0);
+	}
+
+	try {
+		nvobj::transaction::run(pop, [&] {
+			tx_with_callbacks(cb_oncommit_called, cb_onabort_called,
+					  cb_finally_called, cb_work_called);
+		});
+
+		UT_ASSERTeq(cb_work_called, 2);
+		UT_ASSERTeq(cb_oncommit_called, 1);
+		UT_ASSERTeq(cb_onabort_called, 0);
+		UT_ASSERTeq(cb_finally_called, 1);
+	} catch (...) {
+		UT_ASSERT(0);
+	}
+
+	cb_oncommit_called = 0;
+	cb_onabort_called = 0;
+	cb_finally_called = 0;
+	cb_work_called = 0;
+
+	try {
+		nvobj::transaction::run(pop, [&] {
+			tx_with_callbacks(cb_oncommit_called, cb_onabort_called,
+					  cb_finally_called, cb_work_called);
+
+			nvobj::transaction::abort(0);
+		});
+
+		UT_ASSERT(0);
+	} catch (pmem::manual_tx_abort &) {
+		UT_ASSERTeq(cb_work_called, 0);
+		UT_ASSERTeq(cb_oncommit_called, 0);
+		UT_ASSERTeq(cb_onabort_called, 1);
+		UT_ASSERTeq(cb_finally_called, 1);
+	} catch (...) {
+		UT_ASSERT(0);
+	}
+}
+
+void
+test_tx_callback_nested(nvobj::pool<root> &pop)
+{
+	int cb_oncommit_called = 0;
+	int cb_onabort_called = 0;
+	int cb_finally_called = 0;
+	int cb_work_called = 0;
+
+	try {
+		nvobj::transaction::run(pop, [&] {
+			nvobj::transaction::run(pop, [&] {
+				tx_with_callbacks(
+					cb_oncommit_called, cb_onabort_called,
+					cb_finally_called, cb_work_called);
+			});
+
+			UT_ASSERTeq(cb_work_called, 0);
+			UT_ASSERTeq(cb_oncommit_called, 0);
+			UT_ASSERTeq(cb_onabort_called, 0);
+			UT_ASSERTeq(cb_finally_called, 0);
+		});
+
+		UT_ASSERTeq(cb_work_called, 2);
+		UT_ASSERTeq(cb_oncommit_called, 1);
+		UT_ASSERTeq(cb_onabort_called, 0);
+		UT_ASSERTeq(cb_finally_called, 1);
+	} catch (...) {
+		UT_ASSERT(0);
+	}
+}
+
+template <typename T>
+void
+test_tx_callback_scope(nvobj::pool<root> &pop, std::function<void()> commit)
+{
+	int cb_oncommit_called = 0;
+	int cb_onabort_called = 0;
+	int cb_finally_called = 0;
+	int cb_work_called = 0;
+
+	try {
+		{
+			T to(pop);
+
+			tx_with_callbacks(cb_oncommit_called, cb_onabort_called,
+					  cb_finally_called, cb_work_called);
+
+			commit();
+
+			if (std::is_same<T,
+					 nvobj::transaction::manual>::value) {
+				UT_ASSERTeq(cb_work_called, 2);
+				UT_ASSERTeq(cb_oncommit_called, 1);
+			} else {
+				UT_ASSERTeq(cb_work_called, 0);
+				UT_ASSERTeq(cb_oncommit_called, 0);
+			}
+
+			UT_ASSERTeq(cb_onabort_called, 0);
+			UT_ASSERTeq(cb_finally_called, 0);
+		}
+
+		UT_ASSERTeq(cb_work_called, 2);
+		UT_ASSERTeq(cb_oncommit_called, 1);
+		UT_ASSERTeq(cb_onabort_called, 0);
+		UT_ASSERTeq(cb_finally_called, 1);
+	} catch (...) {
+		UT_ASSERT(0);
+	}
+
+	cb_oncommit_called = 0;
+	cb_onabort_called = 0;
+	cb_finally_called = 0;
+	cb_work_called = 0;
+
+	try {
+		counter = 0;
+		{
+			T to(pop);
+
+			tx_with_callbacks(cb_oncommit_called, cb_onabort_called,
+					  cb_finally_called, cb_work_called);
+
+			counter = 1;
+			nvobj::transaction::abort(0);
+
+			UT_ASSERT(0);
+		}
+	} catch (pmem::manual_tx_abort &) {
+		UT_ASSERTeq(cb_work_called, 0);
+		UT_ASSERTeq(cb_oncommit_called, 0);
+		UT_ASSERTeq(cb_onabort_called, 1);
+		UT_ASSERTeq(cb_finally_called, 1);
+	} catch (...) {
+		UT_ASSERT(0);
+	}
+}
+
+void
+test_tx_callback_outiside_tx()
+{
+	try {
+		nvobj::transaction::register_callback(
+			nvobj::transaction::stage::oncommit,
+			[&] { UT_ASSERT(0); });
+
+		UT_ASSERT(0);
+	} catch (pmem::transaction_scope_error &) {
+	} catch (...) {
+		UT_ASSERT(0);
+	}
+}
 }
 
 int
@@ -868,7 +1144,7 @@ main(int argc, char *argv[])
 
 	nvobj::pool<root> pop;
 	try {
-		pop = nvobj::pool<root>::create(path, LAYOUT, POOL_SZIE,
+		pop = nvobj::pool<root>::create(path, LAYOUT, POOL_SIZE,
 						S_IWUSR | S_IRUSR);
 	} catch (...) {
 		UT_FATAL("!pmemobj_create: %s", path);
@@ -891,6 +1167,13 @@ main(int argc, char *argv[])
 
 	test_tx_snapshot(pop);
 
+	test_tx_callback(pop);
+	test_tx_callback_nested(pop);
+	test_tx_callback_scope<nvobj::transaction::manual>(pop, real_commit);
+	test_tx_callback_scope<nvobj::transaction::automatic>(pop, fake_commit);
+
+	test_tx_callback_outiside_tx();
+
 	pop.close();
 
 	return 0;
